/* modified from the template main routine in stk-term project 
Assignment4 -- this is the filter example from Week4 which generates the images in the Week4 slides.
Use it first to see the same 3 channels of output, just to verify that it works and that the output
makes sense.

Then, modify this file according to assignment4's instructions. At that point, you will probably only want 
1 channel of audio.

The goal is to match the impulse response of your tube. 

Don't forget to include "Delay.h" when you add a delayline.
*/

// pick your audio resolution
#define NCHANS 3
#define SRATE_HZ 44100
#define END_SECS 1.0		// ends the performance

#include "WvOut.h"

int
main (void)
{
	Stk::setSampleRate (SRATE_HZ);
	WvOut *outFile = new WvOut ("/zap/test.wav", NCHANS);

	int end = (int) (END_SECS * SRATE_HZ);
	double outFrame[NCHANS];
	int i;
	for (i = 0; i < NCHANS; i++)
		outFrame[i] = 0.0;

/* This code implements the standard difference equation, (Cook: pg. 26)
	y[0] = g * (x[0] + a[1] * x[n - 1]... +a[N] * x[n - N])  
                - b[1] * y[n - 1] - ... -b[M] * y[n - M] 
For simplicity, N = M = "ORDER" */
	 
#define ORDER 1
	int nStates = 1 + ORDER;
	double x[nStates], y[nStates];
	for (i = 0; i < nStates; i++)
		x[i] = y[i] = 0.0;
	/* ------ this is where the real work happens ------ */
	for (int frameCounter = 0; frameCounter < end; frameCounter++)
	{
		// put code here that will produce output more interesting than these zeroes

		double click = (frameCounter == 100) ? 1.0 : 0.0;

		double a[nStates], b[nStates];
		
	/* FIR, one-zero ---------------------- */
		// set FIR coeffs
		double g = 0.5; 		// gain
		a[1] = 1.0; 			// once-delayed
		// set states
		x[0] = click; 			// input
		y[0] = g * (x[0] + a[1] * x[1]);// output
		x[1] = x[0]; 			// once-delayed
		// copy to audio output
		outFrame[0] = x[0];		// filter input to ch0
		outFrame[1] = y[0];		// FIR output to ch1

	/* IIR, one-pole------------------------------- */
		// set extra coeff to make it IIR
		b[1] = -0.5;
		// set states
		y[0] = (g * x[0]) - (b[1] * y[1]);	// output state now is IIR
		y[1] = y[0];				// once-delayed output 
		// copy to audio output 
		outFrame[2] = y[0];			// IIR output to ch2

		// send output
		outFile->tickFrame (outFrame);
	}
	/* ------------------------------------------------ */

	delete outFile;

}


